"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.openFlowReport = exports.loadFlow = exports.collectFlow = exports.persistFlow = void 0;
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const loggin_1 = require("../../../../core/utils/loggin");
const node_process_1 = require("node:process");
// @ts-ignore
const api_1 = require("lighthouse/lighthouse-core/fraggle-rock/api");
const puppeteer = require("puppeteer");
const file_1 = require("../../../../core/utils/file");
const path_1 = require("path");
const dryRun_1 = require("../../../../core/options/dryRun");
const cli_modes_1 = require("../../../../cli-modes");
const budgets_1 = require("../../../assert/utils/budgets");
const user_flow_mock_1 = require("./user-flow.mock");
const open_1 = require("../../options/open");
const interactive_1 = require("../../../../core/options/interactive");
const openFileInBrowser = require("open");
const md_table_1 = require("../../../assert/processes/md-table");
function persistFlow(flow, name, { outPath, format }) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        if (!format.length) {
            format = ['stdout'];
        }
        const jsonReport = yield flow.createFlowResult();
        const results = [];
        if (format.includes('json')) {
            results.push({ format: 'json', out: JSON.stringify(jsonReport) });
        }
        let mdReport = undefined;
        if (format.includes('md')) {
            mdReport = (0, md_table_1.userFlowReportToMdTable)(jsonReport);
            results.push({ format: 'md', out: mdReport });
        }
        if (format.includes('stdout')) {
            mdReport = mdReport || (0, md_table_1.userFlowReportToMdTable)(jsonReport);
            (0, loggin_1.log)(mdReport);
        }
        if (format.includes('html')) {
            const htmlReport = yield flow.generateReport();
            results.push({ format: 'html', out: htmlReport });
        }
        const fileNames = results.map((result) => {
            const fileName = (0, path_1.join)(outPath, `${(0, file_1.toFileName)(name)}.uf.${result.format}`);
            (0, file_1.writeFile)(fileName, result.out);
            return fileName;
        });
        return fileNames;
    });
}
exports.persistFlow = persistFlow;
function collectFlow(cliOption, userFlowProvider) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        let { path, 
        // object containing the LH setting for budgets
        flowOptions: providerFlowOptions, interactions, launchOptions } = userFlowProvider;
        const { config } = providerFlowOptions, rest = (0, tslib_1.__rest)(providerFlowOptions, ["config"]);
        const flowOptions = Object.assign(Object.assign({}, rest), { config: parseUserFlowOptionsConfig(providerFlowOptions.config) });
        // object containing the options for puppeteer/chromium
        launchOptions = launchOptions || {
            headless: false,
            // hack for dryRun => should get fixed inside user flow in future
            defaultViewport: { isMobile: true, isLandscape: false, width: 800, height: 600 }
        };
        // @TODO consider CI vs dev mode => headless, open, persist etc
        if ((0, cli_modes_1.detectCliMode)() !== 'DEFAULT') {
            (0, loggin_1.logVerbose)(`Set headless to true as we are running in ${(0, cli_modes_1.detectCliMode)()} mode`);
            launchOptions.headless = true;
        }
        const browser = yield puppeteer.launch(launchOptions);
        const page = yield browser.newPage();
        (0, loggin_1.logVerbose)(`Collect: ${flowOptions.name} from URL ${cliOption.url}`);
        (0, loggin_1.logVerbose)(`File path: ${(0, path_1.normalize)(path)}`);
        let start = Date.now();
        const flow = !(0, dryRun_1.get)() ? yield (0, api_1.startFlow)(page, flowOptions) : new user_flow_mock_1.UserFlowMock(page, flowOptions);
        // run custom interactions
        yield interactions({ flow, page, browser, collectOptions: cliOption });
        (0, loggin_1.logVerbose)(`Duration: ${flowOptions.name}: ${(Date.now() - start) / 1000}`);
        yield browser.close();
        return flow;
    });
}
exports.collectFlow = collectFlow;
function loadFlow(collect) {
    const { ufPath } = collect;
    const path = (0, path_1.join)((0, node_process_1.cwd)(), ufPath);
    if (!(0, fs_1.existsSync)(path)) {
        throw new Error(`ufPath: ${path} is no directory`);
    }
    let files;
    if ((0, fs_1.lstatSync)(path).isDirectory()) {
        files = (0, fs_1.readdirSync)(path).map(file => (0, path_1.join)(path, file));
    }
    else {
        files = [path];
    }
    const flows = files.filter(f => f.endsWith('js') || f.endsWith('ts'))
        .map((file) => (0, file_1.resolveAnyFile)(file));
    if (flows.length === 0) {
        // @TODO use const for error msg
        throw new Error(`No user flows found in ${ufPath}`);
    }
    return flows;
}
exports.loadFlow = loadFlow;
function openFlowReport(fileNames) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        // open report if requested and not in executed in CI
        if (!(0, dryRun_1.get)() && (0, open_1.get)() && (0, interactive_1.get)()) {
            const htmlReport = fileNames.find(i => i.includes('.html'));
            if (htmlReport) {
                (0, loggin_1.logVerbose)('open HTML report in browser');
                yield openFileInBrowser(htmlReport, { wait: false });
                return Promise.resolve(void 0);
            }
            const mdReport = fileNames.find(i => i.includes('.md'));
            if (mdReport) {
                (0, loggin_1.logVerbose)('open Markdown report in browser');
                yield openFileInBrowser(mdReport, { wait: false });
                return Promise.resolve(void 0);
            }
            const jsonReport = fileNames.find(i => i.includes('.json'));
            if (jsonReport) {
                (0, loggin_1.logVerbose)('open JSON report in browser');
                // @TODO if JSON is given open the file in https://googlechrome.github.io/lighthouse/viewer/
                yield openFileInBrowser(jsonReport, { wait: false });
            }
        }
        return Promise.resolve(void 0);
    });
}
exports.openFlowReport = openFlowReport;
function parseUserFlowOptionsConfig(flowOptionsConfig) {
    var _a, _b;
    flowOptionsConfig = flowOptionsConfig || {};
    // @ts-ignore
    (flowOptionsConfig === null || flowOptionsConfig === void 0 ? void 0 : flowOptionsConfig.extends) || (flowOptionsConfig.extends = 'lighthouse:default');
    // if budgets are given
    if ((_a = flowOptionsConfig === null || flowOptionsConfig === void 0 ? void 0 : flowOptionsConfig.settings) === null || _a === void 0 ? void 0 : _a.budgets) {
        (0, loggin_1.logVerbose)('format given budgets');
        let budgets = (_b = flowOptionsConfig === null || flowOptionsConfig === void 0 ? void 0 : flowOptionsConfig.settings) === null || _b === void 0 ? void 0 : _b.budgets;
        budgets && (budgets = Array.isArray(budgets) ? budgets : (0, budgets_1.readBudgets)(budgets));
        flowOptionsConfig.settings.budgets = budgets;
    }
    return flowOptionsConfig;
}
//# sourceMappingURL=index.js.map